package com.bbc.denadrive

import android.app.PendingIntent
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.addCallback
import androidx.activity.compose.setContent
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.core.net.toUri
import com.bbc.denadrive.apihandlers.ResponsesHandler
import com.bbc.denadrive.apihandlers.makeDriveApiCall
import com.bbc.denadrive.home.ScaffoldWithSidebar
import com.bbc.denadrive.oauth.AuthStatePersistence
import com.bbc.denadrive.oauth.retrieveAuthServConf
import com.bbc.denadrive.ui.theme.DeNaDriveTheme
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationRequest
import net.openid.appauth.AuthorizationResponse
import net.openid.appauth.AuthorizationService
import net.openid.appauth.ResponseTypeValues

class MainActivity : ComponentActivity() {

    private lateinit var responsesHandler: ResponsesHandler
    private var currentPath: String = "root"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        responsesHandler = ResponsesHandler()

        onBackPressedDispatcher.addCallback(this) {
            if (currentPath != "root") makeNewRequest("", false)
            else finish()
        }

        makeNewRequest(parentFolder = "root", forward = true, isFirstRequest = true)

        setContent {
            ScaffoldWithSidebar(
                responsesHandler,
                { name ->
                    Toast.makeText(
                        this,
                        "You pressed the file $name",
                        Toast.LENGTH_SHORT
                    ).show()
                },
                { parent -> makeNewRequest(parent, true) }) {
            }
        }
    }

    private fun makeNewRequest(
        parentFolder: String, forward: Boolean, isFirstRequest: Boolean = false
    ) {
        responsesHandler.lastResponseBody.value = null
        val authPersistence = AuthStatePersistence
        val authState = authPersistence.retrieveAuthState(this)
        val actualFolder: String
        if (!isFirstRequest) {
            if (forward) {
                currentPath += "/$parentFolder"
                actualFolder = parentFolder
            } else {
                currentPath = currentPath.substringBeforeLast("/")
                actualFolder =
                    if (currentPath != "root") currentPath.substringAfterLast("/")
                    else "root"
            }
        } else actualFolder = parentFolder
//        val path = if (parentFolder != "root") "root/$parentFolder" else parentFolder
        Log.e("WAAAT", "makeNewRequest: $actualFolder")
        Log.e("WAAAT", "makeNewRequest: $currentPath")

        val myUrl = "https://www.googleapis.com/drive/v3/files?trashed=false&orderBy=folder"
        makeDriveApiCall(this, authState, myUrl, actualFolder) {
                response, exception ->
            if (exception != null) {
                Log.e("ECCOLO", "${exception.cause}")
                Log.e("ECCOLO", "${exception.message}")
            }
            else {
                Log.e("UOOOO", "$response")
                responsesHandler.newResponseArrived(response)
            }
        }
        authPersistence.putAuthState(this, authState)
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        val resp = AuthorizationResponse.fromIntent(intent)
        val ex = AuthorizationException.fromIntent(intent)
        if (resp != null) {
            Toast.makeText(this, "YES", Toast.LENGTH_SHORT).show()
        }
        else {
            if (ex != null) {
                Toast.makeText(this, "NOPE ${ex.error}", Toast.LENGTH_SHORT).show()
            }
        }

        setContent {
//            val navController = rememberNavController()
//            ScaffoldWithSidebar(navController) {  }
            Greeting("bah")
        }
    }
}

@Composable
fun Greeting(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview() {
    DeNaDriveTheme {
        Greeting("Android")
    }
}