package com.bbc.denadrive.home

import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.foundation.background
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.navigation.NavHostController
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Menu
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.scale
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import kotlinx.coroutines.launch
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.ui.graphics.painter.Painter
import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.ui.res.painterResource
import com.bbc.denadrive.R
import com.bbc.denadrive.apihandlers.MyFile
import com.bbc.denadrive.apihandlers.ResponsesHandler

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ScaffoldWithSidebar(
    responsesHandler: ResponsesHandler,
    onFileClick: (String) -> Unit,
    onFolderClick: (String) -> Unit,
    doIt: () -> Unit,
) {
    // State to control the drawer
    val drawerState = rememberDrawerState(initialValue = DrawerValue.Closed)
    val scope = rememberCoroutineScope()

    // ModalNavigationDrawer
    ModalNavigationDrawer(
        drawerState = drawerState,
        drawerContent = {
            // Drawer content
            Column(
                modifier = Modifier
                    .fillMaxHeight()
                    .fillMaxWidth(0.67f)
                    .background(Color.White)
            ) {
                RectangularButton(backgroundColor = Color.White, text = "Text 1") {  }
//                Spacer(modifier = Modifier.height(10.dp))
                RectangularButton(backgroundColor = Color.White, text = "Text 2") {  }
//                Spacer(modifier = Modifier.height(10.dp))
                RectangularButton(backgroundColor = Color.White, text = "Text 3") {  }
            }
        },
        content = {
            // Main content of the screen
            Scaffold(
                topBar = {
                    TopAppBar(
                        title = { Text("DeNa Drive") },
                        navigationIcon = {
                            IconButton(onClick = {
                                scope.launch {
                                    drawerState.open()
                                }
                            }) {
                                // Icon for the menu (three horizontal lines)
                                Icon(Icons.Filled.Menu, contentDescription = "Menu")
                            }
                        },
                        actions = {
                            IconButton(onClick = { doIt() }) {
                                // Icon for the settings (three vertical dots)
                                Icon(Icons.Filled.MoreVert, contentDescription = "Settings")
                            }
                        }
                    )
                },
                content = { innerPadding ->
                    Box(
                        modifier = Modifier
                            .fillMaxSize()
                            .padding(innerPadding)
                            .padding(16.dp),
                        contentAlignment = Alignment.Center,
                    ) {
                        MyListVisualizer(
                            responsesHandler = responsesHandler,
                            onFileClick = onFileClick,
                            onFolderClick = onFolderClick
                        )
                    }
                }
            )
        }
    )
}

@Composable
fun RectangularButton(
    backgroundColor: Color,
    text: String,
    onClick: () -> Unit
) {
    var isPressed by remember { mutableStateOf(false) }
    val scale by animateFloatAsState(if (isPressed) 0.8f else 1f)

    Box(
        modifier = Modifier
            .fillMaxWidth()
            .height(48.dp) // Set the height of the button
            .background(color = backgroundColor) // Set the background color
            .scale(scale) // Apply the scale effect
            .pointerInput(Unit) {
                // Detect press and release gestures
                detectTapGestures(
                    onPress = {
                        isPressed = true
                        // Wait for the click to be released
                        tryAwaitRelease()
                        isPressed = false
                    },
                    onTap = { onClick() } // Call the onClick action
                )
            }
            .padding(8.dp),
        contentAlignment = Alignment.Center // Center the text
    ) {
        Text(text = text, style = MaterialTheme.typography.bodyLarge) // Button text
    }
}

@Composable
fun MyListVisualizer(responsesHandler: ResponsesHandler,
                     onFileClick: (String) -> Unit,
                     onFolderClick: (String) -> Unit) {
    val fileList = mutableListOf<MyFile>()
    if (responsesHandler.lastResponseBody.value == null) {
        fileList.clear()
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
            Text(text = "Loading...", style = MaterialTheme.typography.titleLarge)
        }
    } else {
        fileList.clear()
        fileList.addAll(
            responsesHandler.lastResponseBody.value!!.toList()
        )
        Box (modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.TopCenter) {
            LazyColumn {
                items(fileList) { file ->
                    val logo = painterResource(id = getId(file.mimeType))
                    val isFolder = file.mimeType.endsWith("folder")
                    val onClick = if (isFolder) onFolderClick
                    else onFileClick
                    LogoTextBox(logo, file.name) { onClick(
                        if (isFolder) file.id else file.name) }
                }
            }
        }
    }
}

fun getId(mimeType: String): Int  {
    val extracted = mimeType.drop("application/vnd.google-apps.".length)
    return when (extracted) {
        "document" -> R.drawable.docslogo_ok
        "presentation" -> R.drawable.slideslogo_ok
        "folder" -> R.drawable.folderlogo_ok
        else -> R.drawable.nuovaicona_
    }
}

@Composable
fun LogoTextBox(
    logoPainter: Painter,
    text: String,
    onClick: () -> Unit) {
    Card(
        modifier = Modifier
            .fillMaxWidth()
            .padding(16.dp)
            .clickable(onClick = onClick),
        shape = RoundedCornerShape(8.dp),
        elevation = CardDefaults.cardElevation(defaultElevation = 4.dp)
    ) {
        Row(
            modifier = Modifier
                .padding(horizontal = 16.dp)
                .fillMaxWidth(),
            verticalAlignment = Alignment.CenterVertically
        ) {
            // Logo on the left
            Image(
                painter = logoPainter,
                contentDescription = "Item Logo",
                modifier = Modifier
                    .size(60.dp) // Adjust size as needed
                    .padding(end = 16.dp) // Space between logo and text
            )

            // Text to the right of the logo
            Text(
                text = text,
                fontSize = 20.sp,
                color = Color.Black
            )
        }
    }
}
