package com.bbc.denadrive.oauth

import android.content.Context
import android.content.SharedPreferences
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKey
import net.openid.appauth.AuthState
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationResponse
import net.openid.appauth.RegistrationResponse
import net.openid.appauth.TokenResponse

object AuthStatePersistence {

    private fun getAuthStatePrefs(context: Context): SharedPreferences {
        val masterKey = MasterKey.Builder(context)
            .setKeyScheme(MasterKey.KeyScheme.AES256_GCM)
            .build()

        return EncryptedSharedPreferences.create(
            context,
            "authState",
            masterKey,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )
    }

    fun isAuthStateSet(context: Context): Boolean {
        return getAuthStatePrefs(context).getString("authState", null) != null
    }

    fun retrieveAuthState(context: Context): AuthState {
        val preferences = getAuthStatePrefs(context)

        preferences.getString("authState", null)?.let {
            return AuthState.jsonDeserialize(it)
        }
        val newAuthState = AuthState(retrieveAuthServConf())
        putAuthState(preferences, newAuthState)
        return newAuthState
    }

    fun putAuthState(context: Context, authState: AuthState) {
        val preferences = getAuthStatePrefs(context)
        putAuthState(preferences, authState)
    }

    private fun putAuthState(preferences: SharedPreferences, authState: AuthState) {
        with (preferences.edit()) {
            this.remove("authState")
            this.putString("authState", authState.jsonSerializeString())
            this.apply()
        }
    }

    fun update(context: Context, authState: AuthState,
               authorizationResponse: AuthorizationResponse?,
               authorizationException: AuthorizationException?) {
        authState.update(authorizationResponse, authorizationException)
        putAuthState(context, authState)
    }

    fun update(context: Context, authState: AuthState,
               tokenResponse: TokenResponse?,
               authorizationException: AuthorizationException?) {
        authState.update(tokenResponse, authorizationException)
        putAuthState(context, authState)
    }

    fun update(context: Context, authState: AuthState,
               registrationResponse: RegistrationResponse?) {
        authState.update(registrationResponse)
        putAuthState(context, authState)
    }
}